"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.attachToElement = void 0;
const sandpack_react_1 = require("@codesandbox/sandpack-react");
const react_1 = __importDefault(require("react"));
const client_1 = require("react-dom/client");
const parse_comment_as_sandbox_options_1 = require("./parse-comment-as-sandbox-options");
const localize_1 = require("./localize");
const attachToElement = (element, options) => {
    let currentRoot;
    let containerElement = null;
    const insert = (node) => {
        if (element.nextSibling === null) {
            element.parentElement?.appendChild(node);
        }
        else {
            element.parentElement?.insertBefore(node, element.nextSibling);
        }
    };
    const exitButton = document.createElement("button");
    exitButton.className = "honkit-plugin-sandpack--exitButton";
    exitButton.textContent = (0, localize_1.t)("exit_console");
    exitButton.setAttribute("style", `
    display: inline-block;
    margin: 0.75em 0.25em;
    font-size: 0.9em;
    border: 0 none;
    border-radius: 3px;
    padding: 0.25em 1em;
    color: white;
    background-color: red;
`);
    exitButton.addEventListener("click", () => exit());
    const runButton = document.createElement("button");
    runButton.className = "honkit-plugin-sandpack--runButton";
    runButton.textContent = (0, localize_1.t)("run_console");
    runButton.setAttribute("style", `
    display: inline-block;
    margin: 0.75em 0.25em;
    font-size: 0.9em;
    border: 0 none;
    border-radius: 3px;
    padding: 0.25em 1em;
    color: white;
    background-color: #5cb85c;
`);
    runButton.addEventListener("click", () => enter());
    const buttonContainer = document.createElement("div");
    buttonContainer.className = "honkit-plugin-sandpack--buttonContainer";
    if (!options.honkitSettings?.hideRunButton) {
        buttonContainer.append(runButton);
    }
    if (!options.honkitSettings?.hideExitButton) {
        buttonContainer.append(exitButton);
    }
    insert(buttonContainer);
    const enter = () => {
        const files = {
            // Workaround: https://github.com/codesandbox/sandpack/issues/373
            "/src/index.js": {
                code: "",
                hidden: true
            },
            ...options.files
        };
        containerElement = document.createElement("div");
        containerElement.classList.add("honkit-plugin-sandpack");
        buttonContainer.parentElement?.insertBefore(containerElement, buttonContainer.nextSibling);
        currentRoot = (0, client_1.createRoot)(containerElement);
        const setup = {
            entry: options.entry,
            dependencies: options.dependencies,
            devDependencies: options.devDependencies,
            environment: "parcel"
        };
        const entry = options.entry;
        const sandpackOptions = options.options;
        console.log({ sandpackOptions });
        const template = (options.template ?? "vanilla");
        currentRoot.render(<sandpack_react_1.Sandpack files={files} customSetup={setup} options={{
                startRoute: entry,
                skipEval: false,
                autorun: true,
                ...sandpackOptions
            }} template={template}/>);
        runButton.style.display = "none";
        exitButton.style.display = "";
    };
    const exit = () => {
        currentRoot?.unmount();
        containerElement?.remove();
        currentRoot = null;
        containerElement = null;
        runButton.style.display = "";
        exitButton.style.display = "none";
    };
    const isOpen = options.honkitSettings?.isOpen ?? false;
    if (isOpen) {
        enter();
    }
    else {
        exit();
    }
};
exports.attachToElement = attachToElement;
function findComments(element) {
    const arr = [];
    for (let i = 0; i < element.childNodes.length; i++) {
        const node = element.childNodes[i];
        if (node.nodeType === 8) {
            arr.push(node);
        }
        else {
            arr.push.apply(arr, findComments(node));
        }
    }
    return arr;
}
function updateCodeBlocs() {
    const getCommentNextPreNode = function (prevNode, nextNode, nextNextNode) {
        if (prevNode && prevNode.nodeName === "PRE") {
            return prevNode;
        }
        if (nextNode && nextNode.nodeName === "PRE") {
            return nextNode;
        }
        else if (nextNextNode && nextNextNode.nodeName === "PRE") {
            // some plugin fallback
            // for https://github.com/azu/gitbook-plugin-include-codeblock
            return nextNextNode;
        }
        return null;
    };
    // <!-- sandpack:{...} -->
    {
        const commentNodes = findComments(document);
        commentNodes
            .filter((commentNode) => {
            return commentNode?.textContent?.trim().startsWith("sandpack:");
        })
            .map((commentNode) => {
            return {
                commentNode,
                options: (0, parse_comment_as_sandbox_options_1.parseCommentAsSandboxOptions)(commentNode?.textContent?.trim())
            };
        })
            .forEach(({ commentNode, options }) => {
            const targetNode = commentNode;
            if (!targetNode) {
                return;
            }
            const prevNode = targetNode.previousElementSibling;
            const nextNode = targetNode.nextElementSibling;
            const nextNextNode = nextNode && nextNode.nextElementSibling;
            const replaceNode = getCommentNextPreNode(prevNode, nextNode, nextNextNode);
            if (replaceNode) {
                // append editor after pre/code
                (0, exports.attachToElement)(replaceNode, options);
            }
            else {
                // replace comment with the editor
                (0, exports.attachToElement)(commentNode, options);
            }
        });
    }
}
// @ts-expect-error: with global
window.gitbook.events.bind("page.change", function () {
    updateCodeBlocs();
});
//# sourceMappingURL=sandpack.jsx.map